package sun.awt.image;

import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.image.RasterFormatException;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.DataBufferInt;
import java.awt.image.DataBuffer;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.image.SampleModel;

public class IntegerInterleavedRaster extends IntegerComponentRaster
{
    private int maxX;
    private int maxY;
    
    public IntegerInterleavedRaster(SampleModel sampleModel, Point point) {
        this(sampleModel, sampleModel.createDataBuffer(), new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public IntegerInterleavedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        this(sampleModel, dataBuffer, new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public IntegerInterleavedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Rectangle rectangle, Point point, IntegerInterleavedRaster integerInterleavedRaster) {
        super(sampleModel, dataBuffer, rectangle, point, integerInterleavedRaster);
        this.maxX = this.minX + this.width;
        this.maxY = this.minY + this.height;
        if (!(dataBuffer instanceof DataBufferInt)) {
            throw new RasterFormatException("IntegerInterleavedRasters must haveinteger DataBuffers");
        }
        DataBufferInt dataBufferInt = (DataBufferInt)dataBuffer;
        this.data = SunWritableRaster.stealData(dataBufferInt, 0);
        if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
            SinglePixelPackedSampleModel singlePixelPackedSampleModel = (SinglePixelPackedSampleModel)sampleModel;
            this.scanlineStride = singlePixelPackedSampleModel.getScanlineStride();
            this.pixelStride = 1;
            (this.dataOffsets = new int[1])[0] = dataBufferInt.getOffset();
            this.bandOffset = this.dataOffsets[0];
            int n = rectangle.x - point.x;
            int n2 = rectangle.y - point.y;
            int[] dataOffsets = this.dataOffsets;
            int n3 = 0;
            dataOffsets[n3] += n + n2 * this.scanlineStride;
            this.numDataElems = singlePixelPackedSampleModel.getNumDataElements();
            this.verify();
            return;
        }
        throw new RasterFormatException("IntegerInterleavedRasters must have SinglePixelPackedSampleModel");
    }
    
    @Override
    public int[] getDataOffsets() {
        return (int[])(int[])this.dataOffsets.clone();
    }
    
    @Override
    public int getDataOffset(int n) {
        return this.dataOffsets[n];
    }
    
    @Override
    public int getScanlineStride() {
        return this.scanlineStride;
    }
    
    @Override
    public int getPixelStride() {
        return this.pixelStride;
    }
    
    @Override
    public int[] getDataStorage() {
        return this.data;
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int[] array;
        if (o == null) {
            array = new int[] { 0 };
        }
        else {
            array = (int[])(int[])o;
        }
        array[0] = this.data[(n2 - this.minY) * this.scanlineStride + (n - this.minX) + this.dataOffsets[0]];
        return array;
    }
    
    @Override
    public Object getDataElements(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int[] array;
        if ((o instanceof int[])) {
            array = (int[])(int[])o;
        }
        else {
            array = new int[n3 * n4];
        }
        for (int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) + this.dataOffsets[0], n6 = 0, i = 0; i < n4; n6 += n3, n5 += this.scanlineStride, ++i) {
            System.arraycopy(this.data, n5, array, n6, n3);
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        this.data[(n2 - this.minY) * this.scanlineStride + (n - this.minX) + this.dataOffsets[0]] = ((int[])(int[])o)[0];
        this.markDirty();
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, Raster raster) {
        int n3 = n + raster.getMinX();
        int n4 = n2 + raster.getMinY();
        int width = raster.getWidth();
        int height = raster.getHeight();
        if (n3 < this.minX || n4 < this.minY || n3 + width > this.maxX || n4 + height > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        this.setDataElements(n3, n4, width, height, raster);
        return;
    }
    
    private void setDataElements(int n, int n2, int w, int n3, Raster raster) {
        if (w <= 0 || n3 <= 0) {
            return;
        }
        int minX = raster.getMinX();
        int minY = raster.getMinY();
        if ((raster instanceof IntegerInterleavedRaster)) {
            IntegerInterleavedRaster integerInterleavedRaster = (IntegerInterleavedRaster)raster;
            int[] dataStorage = integerInterleavedRaster.getDataStorage();
            for (int scanlineStride = integerInterleavedRaster.getScanlineStride(), dataOffset = integerInterleavedRaster.getDataOffset(0), n4 = this.dataOffsets[0] + (n2 - this.minY) * this.scanlineStride + (n - this.minX), j = 0; j < n3; dataOffset += scanlineStride, n4 += this.scanlineStride, ++j) {
                System.arraycopy(dataStorage, dataOffset, this.data, n4, w);
            }
            this.markDirty();
            return;
        }
        Object dataElements = null;
        for (int i = 0; i < n3; ++i) {
            dataElements = raster.getDataElements(minX, minY + i, w, 1, dataElements);
            this.setDataElements(n, n2 + i, w, 1, dataElements);
        }
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int[] array = (int[])(int[])o;
        for (int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) + this.dataOffsets[0], n6 = 0, i = 0; i < n4; n6 += n3, n5 += this.scanlineStride, ++i) {
            System.arraycopy(array, n6, this.data, n5, n3);
        }
        this.markDirty();
        return;
    }
    
    @Override
    public WritableRaster createWritableChild(int n, int n2, int width, int height, int x, int y, int[] array) {
        if (n < this.minX) {
            throw new RasterFormatException("x lies outside raster");
        }
        if (n2 < this.minY) {
            throw new RasterFormatException("y lies outside raster");
        }
        if (n + width < n || n + width > this.minX + this.width) {
            throw new RasterFormatException("(x + width) is outside raster");
        }
        if (n2 + height < n2 || n2 + height > this.minY + this.height) {
            throw new RasterFormatException("(y + height) is outside raster");
        }
        SampleModel sampleModel;
        if (array != null) {
            sampleModel = this.sampleModel.createSubsetSampleModel(array);
        }
        else {
            sampleModel = this.sampleModel;
        }
        return new IntegerInterleavedRaster(sampleModel, this.dataBuffer, new Rectangle(x, y, width, height), new Point(this.sampleModelTranslateX + (x - n), this.sampleModelTranslateY + (y - n2)), this);
    }
    
    @Override
    public Raster createChild(int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        return this.createWritableChild(n, n2, n3, n4, n5, n6, array);
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        if (n <= 0 || n2 <= 0) {
            throw new RasterFormatException("negative " + ((n <= 0) ? "width" : "height"));
        }
        return new IntegerInterleavedRaster(this.sampleModel.createCompatibleSampleModel(n, n2), new Point(0, 0));
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster() {
        return this.createCompatibleWritableRaster(this.width, this.height);
    }
    
    @Override
    public String toString() {
        return new String("IntegerInterleavedRaster: width = " + this.width + " height = " + this.height + " #Bands = " + this.numBands + " xOff = " + this.sampleModelTranslateX + " yOff = " + this.sampleModelTranslateY + " dataOffset[0] " + this.dataOffsets[0]);
    }
}
